#! ruby
# -*- coding: utf-8 -*-

#  This code is for Ruby 2.x and Ruby-GNOME2 2.x.

require './onetweet_glade'

class Onetweet < OnetweetGlade
  INTERVAL_UPDATE_TIMELINE = 120 #second
  INTERVAL_SHOW_NEXT_TWEET = 10 #second
    
  def initialize(path_or_data, root = nil, domain = nil, localedir = nil)
    super
    
    @index_current_tweet = -1
    
    case RUBY_PLATFORM
    when /(mingw|win)/
      @timeline = TweetConsole::Timeline.new( TweetConsole::Timeline::HOME, 
                                              INTERVAL_UPDATE_TIMELINE )
    end
    
    @entry_tweet.modify_font( Pango::FontDescription.new( "Meiryo 10" ) )
    show_next_tweet
    GLib::Timeout.add_seconds( INTERVAL_SHOW_NEXT_TWEET ) do
      show_next_tweet
      true
    end
  end
  
  # signal handlers #

  def on_window_main_destroy(widget, *args)
    Gtk::main_quit
  end

  def on_window_main_delete_event(widget, *args)
    false
  end
  
  # subroutines #
  
  def show_next_tweet
    if @index_current_tweet < ( @timeline.count - 1 )
      @index_current_tweet = @index_current_tweet + 1
      tweet = "[#{@index_current_tweet + 1}/#{@timeline.count}] " + 
              @timeline[@index_current_tweet].userid + ' ' + 
              @timeline[@index_current_tweet].date + ' ' + 
              @timeline[@index_current_tweet].body
      @entry_tweet.text = tweet
    end
  end
end#class Onetweet

class TweetConsole
  def self.home_timeline( count=20 )
    lines = ''
    IO.popen( "C:\\TweetConsole\\twtcnsl.exe /ra #{count}", 'r' ) { |io|
      lines = io.readlines
    }
    lines_to_tweets( lines )
  end

  def self.lines_to_tweets( lines )
    # 取得したデータをTweetの配列に変換
    tweet = nil
    tweets = Array.new
    lines.each do |line|
      # ツイートヘッダなら
      if line.encode( 'utf-8' ) =~ /\w+:\d{4}年\d{1,2}月\d{1,2}日\((日|月|火|水|木|金|土)\)\d{1,2}時\d{1,2}分/#
        tweets << tweet if tweet
        tweet = Tweet.new( line.chomp.split( ':', 2 ) )
      else
        tweet.body << line
      end
    end
    if lines.length > 0 # 最後のツイートも忘れずに
      tweets << tweet if tweet
    end
    tweets.sort! do |a, b|
      a.date <=> b.date
    end
#    puts tweets.length.to_s + "\n"
    return tweets
  end

  class Timeline
    HOME = 0

    include Enumerable
    def each
      @tweets.each do |tweet|
        yield tweet
      end
    end

    def initialize( type, interval_update )
      @type = type
      @interval_update = interval_update
      @tweets = Array.new

      update
      GLib::Timeout.add_seconds( @interval_update ) do
        update
        true
      end
    end
    
    def []( index )
      return @tweets[ index ]
    end
    
    private
    
    def update
      tweets = Array.new
      
      case @type
      when HOME
        tweets = TweetConsole.home_timeline
      end
          
      # 重複チェックしながら表示用タイムラインに追加
      tweets.each do |tweet|
        unless has?(tweet)
          @tweets << tweet
#          puts tweet.userid + " " + tweet.date + "\n" + tweet.body
        end
      end
    end
    
    def has?( tweet )
      @tweets.each do |member|
        return true if tweet === member
      end
      false
    end
  end#class Timeline

  class Tweet
    attr_accessor :userid, :date, :body

    def initialize( items )
      @userid, @date, @body = items
      @userid ||= ""
      @date ||= ""
      @body ||= ""
    end
    
    def ===( other )
      return false unless @userid == other.userid
      return false unless @date == other.date
      return false unless @body == other.body
      true
    end
  end#class Tweet
end#class TweetConsole


# Main program
if __FILE__ == $0
  PROG_PATH = "onetweet.glade"
  PROG_NAME = "onetweet"
  Onetweet.new(PROG_PATH, nil, PROG_NAME)
  Gtk.main
end
